SystemOrganization addCategory: #'Pier-Tests-Book'!

TestCase subclass: #BOEPUBViewTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Tests-Book'!

!BOEPUBViewTest methodsFor: 'book - wiki content' stamp: 'YanniChiu 3/26/2012 01:00'!
bookWithPierWikiContent
	"Answer a Pier book using all the various Pier wiki syntax features"
	| root book chapter |
	root := (PRPage named: 'root')
		title: 'Root';
		contents: 'Root stuff';
		addChild: (book := BOBook named: 'book');
		yourself.
	book addChild: (chapter := BOPortion named: 'chapter-1').
	chapter
		addChild: self exampleParagraphs;
		addChild: self exampleAnnotations;
		addChild: self exampleSections;
		addChild: self exampleLists;
		addChild: self exampleTables;
		addChild: self examplePreformatted;
		addChild: self exampleFormatting;
		addChild: self exampleVerbatim;
		addChild: self exampleLinks;
		addChild: self exampleValues;
		yourself.
	^ root! !

!BOEPUBViewTest methodsFor: 'book - subsections' stamp: 'YanniChiu 3/26/2012 01:16'!
bookWithSubsubsubsections
	"Answer a Pier book using various nested structures"
	| root book part chapter section subsection subsubsection subsubsubsection |
	root := (PRPage named: 'root')
		title: 'Root';
		contents: 'Root stuff';
		addChild: (book := BOBook named: 'book');
		yourself.
	book addChild: (part := BOPortion named: 'part-1').
	part type: #part.
	part addChild: (chapter := BOPortion named: 'chapter-1').
	chapter contents: 'ch1'.
	chapter addChild: (section := BOPortion named: 'section-1-1').
	section contents: 'sec-1-1'.
	section addChild: (subsection := BOPortion named: 'subsection').
	subsection contents: 'subsec'.
	subsection addChild: (subsubsection := BOPortion named: 'subsubsection').
	subsubsection contents: 'subsubsec'.
	subsubsection addChild: (subsubsubsection := BOPortion named: 'subsubsection').
	subsubsubsection contents: 'subsubsubsec'.
	^ root! !

!BOEPUBViewTest methodsFor: 'tests' stamp: 'YanniChiu 3/25/2012 22:48'!
contentsOfStructure: structure
	| stream |
	BOEPUBView new writeStructure: structure on: (stream := ReadWriteStream on: String new).
	^ stream contents! !

!BOEPUBViewTest methodsFor: 'book - empty' stamp: 'YanniChiu 3/26/2012 09:52'!
emptyBook
	| root book chapter |
	root := (PRPage named: 'root')
		title: 'Root';
		contents: 'Root stuff';
		addChild: (book := BOBook named: 'book');
		yourself.
	book addChild: (chapter := BOPortion named: 'chapter').
	^ root! !

!BOEPUBViewTest methodsFor: 'book - wiki content' stamp: 'YanniChiu 3/25/2012 22:40'!
exampleAnnotations
	^ ((BOPortion named: 'annotations')
		contents:
'@@todo todo annotation';
		yourself)! !

!BOEPUBViewTest methodsFor: 'book - wiki content' stamp: 'YanniChiu 3/25/2012 23:05'!
exampleFormatting
	^ ((BOPortion named: 'formatting')
		contents:
'""Formatting""
""bold"" ''italic'' ==monospaced== --strikethrough-- @@subscripted@@ ^^superscripted^^ __underlined__';
		yourself)! !

!BOEPUBViewTest methodsFor: 'book - wiki content' stamp: 'YanniChiu 3/26/2012 00:31'!
exampleLinks
	^ ((BOPortion named: 'links')
		contents:
'""Links""
-*Root>/*
-*Non-existing internal page link>/nonexisting*
-*http://www.foobar.com*
-*Aliased external link>http://www.foobar.com*';
		yourself)! !

!BOEPUBViewTest methodsFor: 'book - wiki content' stamp: 'YanniChiu 3/25/2012 21:53'!
exampleLists
	^ ((BOPortion named: 'lists')
		contents:
'""Numbered List""
#item1
#item2
#item3
""Bulleted List""
-item1
-item2
-item3
""Definition List""
;a
:b
:c
;x
:y
:z';
		yourself)! !

!BOEPUBViewTest methodsFor: 'book - wiki content' stamp: 'YanniChiu 3/25/2012 22:40'!
exampleParagraphs
	^ ((BOPortion named: 'paragraphs')
		contents:
'Lorem ipsum';
		yourself)! !

!BOEPUBViewTest methodsFor: 'book - wiki content' stamp: 'YanniChiu 3/25/2012 23:00'!
examplePreformatted
	^ ((BOPortion named: 'preformatted')
		contents:
'""Preformatted""
=main() { exit(0); }';
		yourself)! !

!BOEPUBViewTest methodsFor: 'book - wiki content' stamp: 'YanniChiu 3/25/2012 22:41'!
exampleSections
	^ ((BOPortion named: 'sections')
		contents:
'!!Level1
!!!!Level2
!!!!!!Level3
!!!!!!!!Level4
!!!!!!!!!!Level5
!!!!!!!!!!!!Level6';
		yourself)! !

!BOEPUBViewTest methodsFor: 'book - wiki content' stamp: 'YanniChiu 3/25/2012 22:55'!
exampleTables
	^ ((BOPortion named: 'tables')
		contents:
'""regular table""
|a|b|c
|x|y|z
""left justified table""
|{asdf|{jklh
|{a|{b
|{xy|{yz;
""right justified table""
|}asdf|}jklh
|}a|}b
|}xy|}yz
""centre justified table""
||asdf||jklh
||a||b
||xy||yz
""table headings""
|!!heading1|!!heading2
|value11|value12
|value21|value22';
		yourself)! !

!BOEPUBViewTest methodsFor: 'book - wiki content' stamp: 'YanniChiu 3/26/2012 00:26'!
exampleValues
	^ ((BOPortion named: 'values')
		contents:
'""Values""
-date: +value:date+
-time: +value:time+
-owner: +value:owner+
-group: +value:group+
-source: +value:source+
-user: +value:user+
-kernel: +value:kernel+';
		yourself)! !

!BOEPUBViewTest methodsFor: 'book - wiki content' stamp: 'YanniChiu 3/26/2012 00:04'!
exampleVerbatim
	^ ((BOPortion named: 'verbatim')
		contents:
'{{{<a href="http://www.foobar.com">verbatim foobar.com link</a>}}}
_
{{{<foobar>}}}invalid tag surrounds this text (verbatim){{{</foobar>}}}
_';
		yourself)! !

!BOEPUBViewTest methodsFor: 'tests' stamp: 'YanniChiu 3/26/2012 10:25'!
testMetadata
	"Use a book that's essentially empty of content, to test the metadata and boilerplate XML"
	| root book epubview zip response |

	root := self emptyBook.
	book := root lookupPath: 'book'.
	epubview := BOEPUBView on: (PRContext new setStructure: book).
	response := BOEPUBMockResponse new.
	epubview respondUsing: response.
	zip := ZipArchive new.
	zip readFrom: response stream.

	self assert: (zip memberNamed: 'mimetype') notNil.
	self assert: (zip memberNamed: 'META-INF/container.xml') notNil.
	self assert: (zip memberNamed: 'OEBPS/content.opf') notNil.
	self assert: (zip memberNamed: 'OEBPS/toc.ncx') notNil.
	self assert: (zip memberNamed: 'OEBPS/book.html') notNil.
	self assert: (zip memberNamed: 'OEBPS/chapter.html') notNil.

	self assert: (zip memberNamed: 'mimetype') contents = 'application/epub+zip'.

	self assert: (zip memberNamed: 'META-INF/container.xml') contents withUnixLineEndings
		= '<?xml version="1.0" encoding="utf-8" standalone="no"?>
<container xmlns="urn:oasis:names:tc:opendocument:xmlns:container" version="1.0"><rootfiles><rootfile full-path="OEBPS/content.opf" media-type="application/oebps-package+xml"/></rootfiles></container>' withUnixLineEndings.

	self assert: (zip memberNamed: 'OEBPS/content.opf') contents withUnixLineEndings
		= '<?xml version="1.0" encoding="utf-8" standalone="no"?>
<package xmlns="http://www.idpf.org/2007/opf" version="2.0" unique-identifier="book">
  <metadata>
    <dc:identifier xmlns:dc="http://purl.org/dc/elements/1.1/" id="book">_idbook</dc:identifier>
    <dc:title xmlns:dc="http://purl.org/dc/elements/1.1/">Book</dc:title>
    <dc:creator xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf" opf:file-as="nil">nil</dc:creator>
    <dc:language xmlns:dc="http://purl.org/dc/elements/1.1/">en</dc:language>
  </metadata>
  <manifest>
    <item id="ncxtoc" href="toc.ncx" media-type="application/x-dtbncx+xml"/>
<item id="id3" href="chapter.html" media-type="application/xhtml+xml"/>
  </manifest>
  <spine toc="ncxtoc">
<itemref idref="id3"/>
  </spine>
</package>' withUnixLineEndings.

	self assert: (zip memberNamed: 'OEBPS/toc.ncx') contents withUnixLineEndings
		= '<?xml version="1.0" encoding="utf-8" standalone="no"?>
<ncx xmlns="http://www.daisy.org/z3986/2005/ncx/" version="2005-1">
  <head>
    <meta name="dtb:uid" content="_idbook"/>
  </head>
  <docTitle>
    <text>Book</text>
  </docTitle>
  <navMap>
<navPoint id="id2" playOrder="2">
  <navLabel><text>Book</text></navLabel>
  <content src="book.html"/>
<navPoint id="id3" playOrder="3">
  <navLabel><text>Chapter</text></navLabel>
  <content src="chapter.html"/>
</navPoint>
</navPoint>
  </navMap>
</ncx>' withUnixLineEndings.

	self assert: (zip memberNamed: 'OEBPS/book.html') contents withUnixLineEndings
		= '<!!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
  <head>
    <title>Book</title>
    <link rel="stylesheet" type="text/css" href="ebook.css"/>
    <meta name="generator" content="Pier Book"/>
  </head>
  <body>
          <div>
            <h1 class="title"><a id="id2"/>Book</h1>
          </div>
          <div>
          </div>
        <hr/>
  </body>
</html>' withUnixLineEndings.

	self assert: (zip memberNamed: 'OEBPS/chapter.html') contents withUnixLineEndings
		= '<!!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
  <head>
    <title>Chapter</title>
    <link rel="stylesheet" type="text/css" href="ebook.css"/>
    <meta name="generator" content="Pier Book"/>
  </head>
  <body>
          <div>
            <h1 class="title"><a id="id3"/>Chapter</h1>
          </div>
          <div>
          </div>
        <hr/>
  </body>
</html>' withUnixLineEndings.
! !

!BOEPUBViewTest methodsFor: 'tests' stamp: 'YanniChiu 3/26/2012 00:59'!
testPierWikiContent
	| root structure epubview stream |

	root := self bookWithPierWikiContent.

	self assert: (self contentsOfStructure: (root lookupPath: 'book/chapter-1/paragraphs'))
		= '<a id="paragraphs"></a><h3>Paragraphs</h3><p>Lorem ipsum</p>'.

	self assert: (self contentsOfStructure: (root lookupPath: 'book/chapter-1/annotations'))
		= '<a id="annotations"></a><h3>Annotations</h3><p class="annotated todo">todo annotation</p>'.

	self assert: (self contentsOfStructure: (root lookupPath: 'book/chapter-1/sections'))
		= '<a id="sections"></a><h3>Sections</h3><h1><a name="83427136"></a>Level1</h1><h2><a name="156851176"></a>Level2</h2><h3><a name="250722448"></a>Level3</h3><h4><a name="208278840"></a>Level4</h4><h5><a name="26898912"></a>Level5</h5><h6><a name="202559112"></a>Level6</h6>'.

	self assert: (self contentsOfStructure: (root lookupPath: 'book/chapter-1/lists'))
		= '<a id="lists"></a><h3>Lists</h3><p><strong>Numbered List</strong></p><ol><li>item1</li><li>item2</li><li>item3</li></ol><p><strong>Bulleted List</strong></p><ul><li>item1</li><li>item2</li><li>item3</li></ul><p><strong>Definition List</strong></p><dl><dt>a</dt><dd>b</dd><dd>c</dd><dt>x</dt><dd>y</dd><dd>z</dd></dl>'.

	self assert: (self contentsOfStructure: (root lookupPath: 'book/chapter-1/tables'))
		= '<a id="tables"></a><h3>Tables</h3><p><strong>regular table</strong></p><table><tr><td>a</td><td>b</td><td>c</td></tr><tr><td>x</td><td>y</td><td>z</td></tr></table><p><strong>left justified table</strong></p><table><tr><td style="text-align: left">asdf</td><td style="text-align: left">jklh</td></tr><tr><td style="text-align: left">a</td><td style="text-align: left">b</td></tr><tr><td style="text-align: left">xy</td><td style="text-align: left">yz;</td></tr></table><p><strong>right justified table</strong></p><table><tr><td style="text-align: right">asdf</td><td style="text-align: right">jklh</td></tr><tr><td style="text-align: right">a</td><td style="text-align: right">b</td></tr><tr><td style="text-align: right">xy</td><td style="text-align: right">yz</td></tr></table><p><strong>centre justified table</strong></p><table><tr><td style="text-align: center">asdf</td><td style="text-align: center">jklh</td></tr><tr><td style="text-align: center">a</td><td style="text-align: center">b</td></tr><tr><td style="text-align: center">xy</td><td style="text-align: center">yz</td></tr></table><p><strong>table headings</strong></p><table><tr><th>heading1</th><th>heading2</th></tr><tr><td>value11</td><td>value12</td></tr><tr><td>value21</td><td>value22</td></tr></table>'.

	self assert: (self contentsOfStructure: (root lookupPath: 'book/chapter-1/preformatted'))
		= '<a id="preformatted"></a><h3>Preformatted</h3><p><strong>Preformatted</strong></p><pre>main() { exit(0); }</pre>'.

	self assert: (self contentsOfStructure: (root lookupPath: 'book/chapter-1/formatting'))
		= '<a id="formatting"></a><h3>Formatting</h3><p><strong>Formatting</strong> <strong>bold</strong> ''italic'' <code>monospaced</code> <del>strikethrough</del> <sub>subscripted</sub> <sup>superscripted</sup> <ins>underlined</ins></p>'.

	self assert: (self contentsOfStructure: (root lookupPath: 'book/chapter-1/verbatim'))
		= '<a id="verbatim"></a><h3>Verbatim</h3><p><a href="http://www.foobar.com">verbatim foobar.com link</a></p><hr/><p><foobar>invalid tag surrounds this text (verbatim)</foobar></p>'.

	self assert: (self contentsOfStructure: (root lookupPath: 'book/chapter-1/links'))
		= '<a id="links"></a><h3>Links</h3><p><strong>Links</strong></p><ul><li><a href="chapter-1.html#links">Links</a></li><li></li><li><a title="http://www.foobar.com" class="external" href="http://www.foobar.com">http://www.foobar.com</a></li><li><a title="http://www.foobar.com" class="external" href="http://www.foobar.com">Aliased external link</a></li></ul>'.

	"Note: these values are not converted correctly in this test, probably due to the mock constext, so this test snippit is somewhat bogus"
	self assert: (self contentsOfStructure: (root lookupPath: 'book/chapter-1/values'))
		= '<a id="values"></a><h3>Values</h3><p><strong>Values</strong></p><ul><li>date: date</li><li>time: time</li><li>owner: owner</li><li>group: group</li><li>source: source</li><li>user: user</li><li>kernel: kernel</li></ul>'.
! !

!BOEPUBViewTest methodsFor: 'tests' stamp: 'YanniChiu 3/26/2012 01:19'!
testSubsubsubsections
	| root book epubview zip response content section |

	root := self bookWithSubsubsubsections.
	book := root lookupPath: 'book'.
	epubview := BOEPUBView on: (PRContext new setStructure: book).
	response := BOEPUBMockResponse new.
	epubview respondUsing: response.
	zip := ZipArchive new.
	zip readFrom: response stream.

	self assert: (zip memberNamed: 'OEBPS/book.html') notNil.
	self assert: (zip memberNamed: 'OEBPS/part-1.html') notNil.
	self assert: (zip memberNamed: 'OEBPS/chapter-1.html') notNil.

	self assert: (self contentsOfStructure: (root lookupPath: 'book/part-1/chapter-1'))
		= '<p>ch1</p><a id="section-1-1"></a><h3>Section-1-1</h3><p>sec-1-1</p><p>subsec</p><p>subsubsec</p><p>subsubsubsec</p>'.
! !

!BOEPUBViewTest methodsFor: 'tests' stamp: 'YanniChiu 3/26/2012 00:44'!
testZipMembers
	| root book epubview zip response content section |

	root := self bookWithPierWikiContent.
	book := root lookupPath: 'book'.
	epubview := BOEPUBView on: (PRContext new setStructure: book).
	response := BOEPUBMockResponse new.
	epubview respondUsing: response.
	zip := ZipArchive new.
	zip readFrom: response stream.

	self assert: (zip memberNamed: 'OEBPS/book.html') notNil.
	self assert: (zip memberNamed: 'OEBPS/chapter-1.html') notNil.
	self assert: (section := zip memberNamed: 'OEBPS/paragraphs.html') notNil.
	self assert: (section := zip memberNamed: 'OEBPS/annotations.html') notNil.
	self assert: (section := zip memberNamed: 'OEBPS/sections.html') notNil.
	self assert: (section := zip memberNamed: 'OEBPS/lists.html') notNil.
	self assert: (section := zip memberNamed: 'OEBPS/preformatted.html') notNil.
	self assert: (section := zip memberNamed: 'OEBPS/formatting.html') notNil.
	self assert: (section := zip memberNamed: 'OEBPS/verbatim.html') notNil.
	self assert: (section := zip memberNamed: 'OEBPS/links.html') notNil.
! !

!BOEPUBViewTest methodsFor: 'tests' stamp: 'YanniChiu 3/26/2012 00:40'!
todo
	"
	- formatting: italic formatting does not translate
	- missing <hr/> when it's the last item on page
	- links: *Email>info@foobar.com* is not consistently converted
	"! !

Object subclass: #BOEPUBMockResponse
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Tests-Book'!

!BOEPUBMockResponse methodsFor: 'accessing' stamp: 'YanniChiu 3/7/2012 11:04'!
attachmentWithFileName: ignored
	"Do nothing"! !

!BOEPUBMockResponse methodsFor: 'accessing' stamp: 'YanniChiu 3/7/2012 11:04'!
binary
	"Do nothing"! !

!BOEPUBMockResponse methodsFor: 'accessing' stamp: 'YanniChiu 3/7/2012 11:04'!
cacheForever
	"Do nothing"! !

!BOEPUBMockResponse methodsFor: 'accessing' stamp: 'YanniChiu 3/7/2012 11:04'!
contentType: ignored
	"Do nothing"! !

!BOEPUBMockResponse methodsFor: 'accessing' stamp: 'YanniChiu 3/7/2012 11:04'!
stream

	^ stream ifNil: [ stream := (RWBinaryOrTextStream on: String new) binary ]! !

!BOEPUBMockResponse methodsFor: 'accessing' stamp: 'YanniChiu 3/7/2012 11:04'!
stream: anObject

	stream := anObject! !
